;;
;; configuraiton.scm
;;
;; Configuration parameters used by various modules.
;;
;; ISC License
;;
;; Copyright 2023 Brmlab, z.s.
;; Dominik Pantůček <dominik.pantucek@trustica.cz>
;;
;; Permission to use, copy, modify, and/or distribute this software
;; for any purpose with or without fee is hereby granted, provided
;; that the above copyright notice and this permission notice appear
;; in all copies.
;; 
;; THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
;; WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
;; WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
;; AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR
;; CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
;; OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
;; NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
;; CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
;;

(declare (unit configuration))

(module
 configuration
 (
  *current-month*
  *member-file-context*
  *member-file-check-syntax*
  *member-default-joined*
  *member-suspend-max-months*
  )

 (import scheme
	 (chicken base)
	 (chicken time)
	 (chicken time posix)
	 month)

 ;; Current month - if changed, we get the actual state for given month.
 (define *current-month*
   (make-parameter
    (let ((d (seconds->local-time (current-seconds))))
      (list (+ 1900 (vector-ref d 5))
	    (+ (vector-ref d 4) 1)))))

 ;; Configuration of error reporting
 (define *member-file-context* (make-parameter 3))

 ;; Tolerance to formal errors (invalid key or key without value):
 ;;  'error - show source and exits with error
 ;;  'warning - show source and error, continue
 ;;  'quiet - ignore
 (define *member-file-check-syntax* (make-parameter 'error))

 ;; Default as specified by the original implementation if the 'joined
 ;; key is missing in member file.
 (define *member-default-joined* (make-parameter (make-month 2015 1)))

 ;; How long the member can be suspended without any action required?
 (define *member-suspend-max-months* (make-parameter 24))

 )
